﻿namespace TabsStudioSQLComment
{
    public class SQLComment : TabsStudioExt.ITabsStudioAddin, TabsStudioExt.IConfigurable
    {
        public void OnConnection(TabsStudioExt.ITabsStudioEngine engine_, EnvDTE80.DTE2 dte)
        {
            engine = engine_;
            engine.TabCreated += OnTabCreated;
            engine.TabRenamed += OnTabRenamed;
            engine.TabExtensionCreated += OnTabExtensionCreated;
            engine.TabExtensionDestroyed += OnTabExtensionDestroyed;
            engine.TabExtensionRenamed += OnTabExtensionRenamed;

            storage = new OptionsStorage(engine.GetAddinSettingsDirectory());
            options = storage.Load();
        }

        public void OnDisconnection()
        {
            engine.TabCreated -= OnTabCreated;
            engine.TabRenamed -= OnTabRenamed;
            engine.TabExtensionCreated -= OnTabExtensionCreated;
            engine.TabExtensionDestroyed -= OnTabExtensionDestroyed;
            engine.TabExtensionRenamed -= OnTabExtensionRenamed;
        }

        public void Configure(System.Windows.Window parent)
        {
            OptionsDialog dialog = new OptionsDialog(options);
            dialog.Owner = parent;
            dialog.WindowStartupLocation = System.Windows.WindowStartupLocation.CenterOwner;
            if (dialog.ShowDialog().GetValueOrDefault())
                storage.Save(options);
        }

        private void OnTabCreated(object sender, TabsStudioExt.TabEventArgs e)
        {
            UpdateComment(e.Tab.TitleManager, e.Tab.TabExtensions[0].Window);
        }

        private void OnTabRenamed(object sender, TabsStudioExt.TabEventArgs e)
        {
            if (e.Tab.TabExtensions.Count == 1)
                UpdateComment(e.Tab.TitleManager, e.Tab.TabExtensions[0].Window);
        }

        private void OnTabExtensionDestroyed(object sender, TabsStudioExt.TabExtensionEventArgs e)
        {
            if (e.Tab.TabExtensions.Count == 2)
            {
                TabsStudioExt.TabExtension remainingExtension = e.Tab.TabExtensions[0].Window == e.TabExtension.Window ? 
                        e.Tab.TabExtensions[1] : e.Tab.TabExtensions[0];
                if (IsPoorlyNamedTab(e.Tab.TitleManager) || IsPoorlyNamedTab(remainingExtension.TitleManager))
                    AddComment(e.Tab.TitleManager, remainingExtension.Window);
            }
        }

        private void OnTabExtensionCreated(object sender, TabsStudioExt.TabExtensionEventArgs e)
        {
            if (e.Tab.TabExtensions.Count == 2)
                RemoveComment(e.Tab.TitleManager);
            UpdateComment(e.TabExtension.TitleManager, e.TabExtension.Window);
        }

        private void OnTabExtensionRenamed(object sender, TabsStudioExt.TabExtensionEventArgs e)
        {
            UpdateComment(e.TabExtension.TitleManager, e.TabExtension.Window);
        }

        private void UpdateComment(TabsStudioExt.ITitleManager titleManager, EnvDTE.Window w)
        {
            RemoveComment(titleManager);
            if (IsPoorlyNamedTab(titleManager))
                AddComment(titleManager, w);
        }

        private void AddComment(TabsStudioExt.ITitleManager titleManager, EnvDTE.Window w)
        {
            try
            {
                System.Text.RegularExpressions.Regex regex =
                    new System.Text.RegularExpressions.Regex(options.commentRegex,
                        System.Text.RegularExpressions.RegexOptions.Multiline);
                System.Text.RegularExpressions.Match match =
                    regex.Match(GetDocumentText(w.Document));

                string comment = "";
                foreach (System.Text.RegularExpressions.Capture c in match.Groups["M"].Captures)
                    comment += c.Value;
                comment = comment.Replace("\r", "").Replace("\n", "");

                if (comment.Length > 0)
                    AddComment(titleManager, comment);
            }
            catch (System.Exception)
            {
            }
        }

        private bool IsPoorlyNamedTab(TabsStudioExt.ITitleManager titleManager)
        {
            System.Text.RegularExpressions.Regex regex =
                new System.Text.RegularExpressions.Regex(@"SQLQuery\d+\.sql");
            return regex.IsMatch(titleManager.UntransformedTitle);
        }

        // throws System.Exception
        private string GetDocumentText(EnvDTE.Document doc)
        {
            EnvDTE.TextDocument textDoc = doc.Object("TextDocument") as EnvDTE.TextDocument;
            if (textDoc == null)
                return "";
            return textDoc.StartPoint.CreateEditPoint().GetText(textDoc.EndPoint);
        }

        private void AddComment(TabsStudioExt.ITitleManager titleManager, string comment)
        {
            titleManager.Transforms.Add(
                new TabsStudioExt.TitleTransform("^.*$", "$_ " + comment, transformTag));
            titleManager.ApplyTransforms();
        }

        private void RemoveComment(TabsStudioExt.ITitleManager titleManager)
        {
            foreach (TabsStudioExt.TitleTransform t in titleManager.Transforms)
            {
                if (t.Id == transformTag)
                {
                    titleManager.Transforms.Remove(t);
                    titleManager.ApplyTransforms();
                    return;
                }
            }
        }

        private static string transformTag = "[SQLComment]";
        private TabsStudioExt.ITabsStudioEngine engine;
        private OptionsStorage storage;
        private Options options;
    }
}
